# This constructs a minimal initramfs userspace containing `signify`,
# `lvm`, `dmsetup`, `cryptsetup`, and `kexec`, plus the
# modules from the `kernel` derivation, and creates an
# initramfs-formatted `cpio` archive from their contents.
{ lib
, nixpkgsOnBuildForHost
, nixpkgsOnBuildForBuild
, bootScript ? ../boot.sh   # symlinked to /init if non-null

# see six-initrd
, contents ? {}
, kernel ? null
, modules ? []
, symlinkUsrToRoot ? true
, symlinkSbinToBin ? true

, six-initrd

, withSignify ? true
# Inclusion of these is temporarily disabled.  I had a bunch of
# ugly space-saving hacks that were removed during the pre-release
# cleanup, and it turns out that without those hacks there isn't enough
# space for the large userspace tools.  Once I clean up and
# reinstate those hacks I will reenable this.
, withLvm ? false
, withCryptsetup ? false
, withKexec ? false
, withNvramTool ? false # nixpkgsOnBuildForHost.stdenv.hostPlatform.isx86
}:

let
  inherit (nixpkgsOnBuildForHost.pkgsStatic) signify lvm2 cryptsetup kexec-tools;
  nvramtool = nixpkgsOnBuildForHost.pkgsStatic.nvramtool.overrideAttrs(a: {
    NIX_CFLAGS_COMPILE = "-D__GLIBC__";
  });

in
six-initrd.override {
  inherit lib;
  pkgsForBuild = nixpkgsOnBuildForBuild;
  pkgsForHost = nixpkgsOnBuildForHost;
  inherit kernel modules;
  contents = lib.optionalAttrs withSignify {
    "bin/signify" = "${signify}/bin/signify";
  } // lib.optionalAttrs withLvm {
    "bin/lvm" = "${lib.getBin lvm2}/bin/lvm";
    "bin/dmsetup" = "${lib.getBin lvm2}/bin/dmsetup";
  } // lib.optionalAttrs withCryptsetup {
    "bin/cryptsetup" = "${cryptsetup}/bin/cryptsetup";
  } // lib.optionalAttrs withKexec {
    "bin/kexec" = "${kexec-tools}/bin/kexec";
  } // lib.optionalAttrs withNvramTool {
    "bin/nvramtool" = "${nvramtool}/bin/nvramtool";
  } // lib.optionalAttrs (bootScript != null) {
    "sbin/init" = "${bootScript}";
  } // contents;
}
